/*
 * FreeModbus Libary: Atmel AT91SAM3S Demo Application
 * Copyright (C) 2010 Christian Walter <cwalter@embedded-solutions.at>
 *
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *   notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *   notice, this list of conditions and the following disclaimer in the
 *   documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *   derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * IF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * File: $Id: demo.c,v 1.2 2010/06/06 13:46:42 wolti Exp $
 */


/*----------------------------------------------------------------------------
  Headers
----------------------------------------------------------------------------*/
#include "board.h"
/* ----------------------- Qtouch includes ----------------------------------*/
#include "libqtouch.h"
#include "atmel_app.h"
/* ----------------------- Modbus includes ----------------------------------*/
#include "mb.h"
#include "mbport.h"



/*----------------------------------------------------------------------------
 *        Local definitions
 *----------------------------------------------------------------------------*/
#define REG_INPUT_START                 ( 1000 )
#define REG_INPUT_NREGS                 ( 64 )

#define REG_HOLDING_START               ( 1 )
#define REG_HOLDING_NREGS               ( 32 )

///** Reference voltage for ADC,in mv*/
//#define VOLT_REF   (3300)
//
///* The maximal digital value*/
//#define MAX_DIGITAL (4095)
//
///** channel connected to internal Temperature Sensor */
//#define ADC_TEMP_SENSOR_CHANNEL     ADC_CHANNEL_15 
//
///** channel connected to on-board potentiometer*/
//#define ADC_POT_CHANNEL             ADC_CHANNEL_5
//
///** RS485 Mode needs Board modification and RS845 <--> RS232 transceiver. See
//Application note for details */
////#define RS485 1
//////#define RS232 1
//
//#define GET_SENSOR_STATE(SENSOR_NUMBER) (qt_measure_data.qt_touch_status.sensor_states[(SENSOR_NUMBER/8)] & (1 << (SENSOR_NUMBER % 8)))
//#define GET_ROTOR_SLIDER_POSITION(ROTOR_SLIDER_NUMBER) qt_measure_data.qt_touch_status.rotor_slider_values[ROTOR_SLIDER_NUMBER]


/*----------------------------------------------------------------------------
                                extern variables
----------------------------------------------------------------------------*/
///* This configuration data structure parameters if needs to be changed will be
//   changed in the qt_set_parameters function */
//extern qt_touch_lib_config_data_t qt_config_data;
///* measurement data */
//extern qt_touch_lib_measure_data_t qt_measure_data;
///* Get sensor delta values */
//extern int16_t qt_get_sensor_delta( uint8_t sensor);
///* Output can be observed in the watch window using this pointer */
//qt_touch_lib_measure_data_t *pqt_measure_data = &qt_measure_data;
//
///* QDebug */
//extern uint16_t timestamp1_hword;
//extern uint16_t timestamp1_lword;
//extern uint16_t timestamp2_hword;
//extern uint16_t timestamp2_lword;
//extern uint16_t timestamp3_hword;
//extern uint16_t timestamp3_lword;

/*----------------------------------------------------------------------------
 *        Local variables
 *----------------------------------------------------------------------------*/
///** time stamp */
//uint32_t time_stamp = 0;
///* Timer period in msec. */
//uint16_t qt_measurement_period_msec = 25u;
//
///*status flags to indicate the re-burst for library*/
//uint16_t status_flag = 0u;
//uint16_t burst_flag = 0u;
//
//uint8_t vld_pressed = 0;
//uint8_t up_pressed = 0;
//uint8_t dwn_pressed = 0;
//uint8_t lft_pressed = 0;
//uint8_t rgt_pressed = 0;

/*----------------------------------------------------------------------------
  static variables
----------------------------------------------------------------------------*/
/* Modbus */
static uint16_t   usRegInputStart = REG_INPUT_START;
static uint16_t   usRegInputBuf[REG_INPUT_NREGS];
static uint16_t   usRegHoldingStart = REG_HOLDING_START;
static uint16_t   usRegHoldingBuf[REG_HOLDING_NREGS];

///* Qtouch */
///* flag set by timer ISR when it's time to measure touch */
//static volatile uint8_t time_to_measure_touch = 0u;
//
///* current time, set by timer ISR */
//static volatile uint16_t current_time_ms_touch = 0u;



/*----------------------------------------------------------------------------
 *        Local functions
 *----------------------------------------------------------------------------*/
/* initialise host app, pins, watchdog, etc */
//static void init_system( void );

/* configure timer ISR to fire regularly */
static void init_timer_isr( void );

/*  Assign the parameters values to global configuration parameter structure    */
static void qt_set_parameters( void );

/* Configure the sensors */
static void config_sensors(void);

///* Prototypes */
//static void _AdcInit();
//static void _SetupHardware( void );
//static void _AdcTempSensorInit( void );
//static uint16_t _AdcReadTempSensor( void );
//static uint16_t _AdcReadAnalogInput( void ) ;
//static void _AdcPotentiometerInit( void ) ;
//static void _IoLed( uint8_t ubIdx, uint8_t bTurnOn );
//static void _ConfigureLeds( void ) ;
//static void _QtouchSliderInit ( void );
//static uint8_t _ReadQtouchSlider ( void );
//uint16_t Read_Temperature(void);
/* Prototypes */
extern void _AdcInit();
extern void _SetupHardware( void );
extern void _AdcTempSensorInit( void );
extern uint16_t _AdcReadTempSensor( void );
extern uint16_t _AdcReadAnalogInput( void ) ;
extern void _AdcPotentiometerInit( void ) ;
extern void _IoLed( uint8_t ubIdx, uint8_t bTurnOn );
extern void _ConfigureLeds( void ) ;
extern void _QtouchSliderInit ( void );
extern uint8_t _ReadQtouchSlider ( void );
uint16_t Read_Temperature(void);

/**
 *  \brief Application entry point.
 *
 *
 *  \return Unused (ANSI-C compatibility).
 *  \callgraph
 */
int
main( void )
{
    
         
    /* Modbus Hardware Init */
    _SetupHardware(  ) ;
    
    /* Initialize ADC */
    _AdcInit( ) ;
    
    /* Temperature Sensor Channel Init */
    _AdcTempSensorInit();
    
    /* AD5 for Potentiometer Channel Init */    
    _AdcPotentiometerInit( ) ;

    /* Configure PIO for blue and Green LEDs */    
    _ConfigureLeds( ) ;
    
    /* Init of the Qtouch Slider */
    _QtouchSliderInit( ) ;   

    const uint8_t     ucSlaveID[] = { 0xAA, 0xBB, 0xCC };
    eMBErrorCode    eStatus;

    for( ;; )
    {
        if( MB_ENOERR != ( eStatus = eMBInit( MB_RTU, 0x0a, 1, 19200, MB_PAR_EVEN ) ) )
        {
            /* Can not initialize. Add error handling code here. */
        }
        else
        {      
            if( MB_ENOERR != ( eStatus = eMBSetSlaveID( 0x34, TRUE, ucSlaveID, 3 ) ) )
            {
                /* Can not set slave id. Check arguments */
            }
            else if( MB_ENOERR != ( eStatus = eMBEnable(  ) ) )
            {
                /* Enable failed. */
            }
            else
            {      
                usRegHoldingBuf[0] = 1;
                do
                {
                    ( void )eMBPoll(  );
                    
                    /* Modbus Read Registers from master */
                    usRegInputBuf[1] = (uint16_t)Read_Temperature();
                    usRegInputBuf[2] = _AdcReadAnalogInput();
                    usRegInputBuf[3] = _ReadQtouchSlider( ) ;
                    
                   /* Modbus Write Regusters from master */
                    _IoLed( (uint8_t)usRegHoldingBuf[1],(uint8_t)usRegHoldingBuf[2] ) ;
                     
                }
                while( usRegHoldingBuf[0] );
                ( void )eMBDisable(  );
                ( void )eMBClose(  );                
            }
        }
    }    
    return 1;
}

static void _SetupHardware( void )
{
    WDT_Disable( WDT );
    
    uint32_t i = 0;
    for( i = 0; i < 35; i++ )
    {
        NVIC_SetPriority( (IRQn_Type)i, 0xF << 4 ) ;
    }    
}

//static void _AdcInit()
//{
//    /* Initialize ADC */
//    ADC_Initialize( ADC, ID_ADC );
//    
//    /* Enable Freerun mode for start conversion erratum */
//    ADC_CfgTrigering( ADC, 0,0, 1 ) ;
//
//    /*  startup = 8:    512 periods of ADCClock
//     * for prescal = 4
//     *     prescal: ADCClock = MCK / ( (PRESCAL+1) * 2 ) => 64MHz / ((4+1)*2) = 6.4MHz
//     *     ADC clock = 6.4 MHz
//     */
//    ADC_cfgFrequency( ADC, 8, 4 );
//
//    ADC_check( ADC, BOARD_MCK );
//  
//}
//
//
//static void _AdcTempSensorInit( void )
//{
//
//    /*Enable ADC channel for the temperature sensor*/
//    ADC_EnableChannel( ADC,  ADC_TEMP_SENSOR_CHANNEL ) ;
//
//    /*Enable the temperature sensor*/
//    ADC_EnableTS( ADC, ADC_ACR_TSON ) ;
//    
//    /*Enable the temperature sensor*/    
//    ADC_EnableIt(ADC, 0x1 << ADC_TEMP_SENSOR_CHANNEL ) ;
//  
//}
//
//static void _AdcPotentiometerInit( void ) 
//{
//
//    /*Enable  channel for potentiometer*/
//    ADC_EnableChannel( ADC, ADC_POT_CHANNEL ) ;
//    ADC_EnableIt(ADC, 0x1 << ADC_CHANNEL_5 ) ;    
// 
//}
//
//
//static uint16_t _AdcReadTempSensor( void )
//{
//    unsigned int status ;
//  
//    status = ADC_GetStatus(ADC);
//              
//    while ( (status & ADC_ISR_EOC15)!=ADC_ISR_EOC15)
//    {
//      status = ADC_GetStatus(ADC);
//    }
//    
//    return ( ADC_GetConvertedData( ADC, ADC_CHANNEL_15) ) ;    
//}
//
//static uint16_t _AdcReadAnalogInput( void )
//{
//    unsigned int status ;
//  
//    status = ADC_GetStatus(ADC);
//              
//    while ( (status & ADC_ISR_EOC5)!=ADC_ISR_EOC5)
//    {
//      status = ADC_GetStatus(ADC);
//    }
//    
//    return ( ADC_GetConvertedData( ADC, ADC_CHANNEL_5) ) ;    
//}
//
//static void _IoLed( uint8_t ubIdx1, uint8_t ubIdx2 )
//{
//    // Manage BLUE LED
//    if ( ubIdx1 )
//    {
//      LED_Set( 0 );
//    }
//    else
//    {
//      LED_Clear( 0 );
//    }
//
//    // Manage GREEN LED
//    if ( ubIdx2 )
//    {
//      LED_Set( 1 );
//    }
//    else
//    {
//      LED_Clear( 1 );
//    }
// 
//}
//
//static void _QtouchSliderInit ( void )
//{
//    /* Qtouch Hardware Init */
//    /* reset touch sensing */
//    qt_reset_sensing();
//
//    /* Configure the Sensors as keys or Keys With Rotor/Sliders in this function */
//    config_sensors();
//
//    /* initialise touch sensing */
//    qt_init_sensing();
//
//    /* Set the parameters like recalibration threshold, Max_On_Duration etc in this function by the user */
//    qt_set_parameters( );
//
//    /* configure timer ISR to fire regularly */
//    init_timer_isr();
//
//    /* Address to pass address of user functions */
//    /* This function is called after the library has made capacitive measurements,
//    * but before it has processed them. The user can use this hook to apply filter
//    * functions to the measured signal values.(Possibly to fix sensor layout faults) */
//    qt_filter_callback = 0;  
//    
//}
//
//uint8_t _ReadQtouchSlider ( void )
//{
//
//
//    static uint8_t old_position = 0;
//    if( time_to_measure_touch )
//    {
//      /* clear flag: it's time to measure touch */
//      time_to_measure_touch = 0u;
//      do
//      {
//        /*  one time measure touch sensors    */
//        status_flag = qt_measure_sensors( current_time_ms_touch );
//        burst_flag = status_flag & QTLIB_BURST_AGAIN;
//        /*Time critical host application code goes here*/
//      } while (burst_flag);
//    }
//
//      /*  Time Non-critical host application code goes here  */
//    if( GET_ROTOR_SLIDER_POSITION(0) != old_position )
//    {
//      old_position = GET_ROTOR_SLIDER_POSITION(0);
//    }
//    return ( old_position );
//}
//
///**
// *  \brief Configure LEDs
// *
// *  Configures LEDs \#1 and \#2 (cleared by default).
// */
//static void _ConfigureLeds( void )
//{
//    LED_Configure( 0 ) ;
//    LED_Configure( 1 ) ;
//}
//
//static void qt_set_parameters( void )
//{
//    /*  This will be modified by the user to different values   */
//    qt_config_data.qt_di              = DEF_QT_DI;
//    qt_config_data.qt_neg_drift_rate  = DEF_QT_NEG_DRIFT_RATE;
//    qt_config_data.qt_pos_drift_rate  = DEF_QT_POS_DRIFT_RATE;
//    qt_config_data.qt_max_on_duration = DEF_QT_MAX_ON_DURATION;
//    qt_config_data.qt_drift_hold_time = DEF_QT_DRIFT_HOLD_TIME;
//    qt_config_data.qt_recal_threshold = DEF_QT_RECAL_THRESHOLD;
//    qt_config_data.qt_pos_recal_delay = DEF_QT_POS_RECAL_DELAY;
//}
///**
// *  \brief Configure the sensors
// *
// *      PIOC     0  1  chanel 0
// *      PIOC     2  3  chanel 1
// *      PIOC     4  5  chanel 2
// *      PIOC     6  7  chanel 3
// *      PIOC     8  9  chanel 4
// *      PIOC    10 11  chanel 5
// *      PIOC    12 13  chanel 6
// *      PIOC    14 15  chanel 7
// *      PIOC    16 17  chanel 8
// *      PIOC    18 19  chanel 9
// *      PIOC    20 21  chanel 10
// *      PIOA     0  1  chanel 16   : slider
// *      PIOA     2  3  chanel 17   : slider
// *      PIOA     4  5  chanel 18   : slider
// *      PIOA     6  7  chanel 19
// *      PIOA     8  9  chanel 20
// *      PIOA    10 11  chanel 21
// *      PIOA    12 13  chanel 22
// *      PIOA    14 15  chanel 23
// *      PIOA    16 17  chanel 24
// *      PIOA    18 19  chanel 25
// *      PIOA    20 21  chanel 26
// *      PIOA    22 23  chanel 27
// *      PIOA    24 25  chanel 28
// *      PIOA    26 27  chanel 29
// *      PIOA    28 29  chanel 30
// *      PIOA    30 31  chanel 31
// *
// */
//static void config_sensors(void)
//{
//    PMC_EnablePeripheral(ID_PIOA);
//    PMC_EnablePeripheral(ID_PIOC);
//
//    /* Slider: PA0, PA1, PA2, PA3, PA4, PA5 */
//    qt_enable_slider( CHANNEL_16, CHANNEL_18, AKS_GROUP_1, 16u, HYST_6_25, RES_8_BIT, 0u  );
//}
//
//
//
////*--------------------------------------------------------------------------------------
////* Function Name       : Read_Temperature
////* Object              : 
////* Input Parameters    : none.
////* Output Parameters   : temperature conversion.
////*--------------------------------------------------------------------------------------
//uint16_t Read_Temperature(void)
//{
//  return ( _AdcReadTempSensor());  
//}
//

/**
 *  \brief  Handler for Sytem Tick interrupt.
 *
 *  Set systick event flag
 */
//void SysTick_Handler(void)
//{
//    /* set flag: it's time to measure touch */
//    time_to_measure_touch = 1u;
//
//    /* update the current time */
//    current_time_ms_touch += qt_measurement_period_msec;
//}
//
///**
// *  \brief  configure timer ISR to fire regularly
// *
// *  Configure Timer Counter 0 to generate an interrupt every 25ms.
// */
//static void init_timer_isr( void )
//{
//    SysTick_Config(BOARD_MCK / (1000 / qt_measurement_period_msec));
//}



eMBErrorCode
eMBRegInputCB( uint8_t * pucRegBuffer, uint16_t usAddress, uint16_t usNRegs )
{
    eMBErrorCode    eStatus = MB_ENOERR;
    int             iRegIndex;

    if( ( usAddress >= REG_INPUT_START )
        && ( usAddress + usNRegs <= REG_INPUT_START + REG_INPUT_NREGS ) )
    {
        iRegIndex = ( int )( usAddress - usRegInputStart );
        while( usNRegs > 0 )
        {
            *pucRegBuffer++ =
                ( unsigned char )( usRegInputBuf[iRegIndex] >> 8 );
            *pucRegBuffer++ =
                ( unsigned char )( usRegInputBuf[iRegIndex] & 0xFF );
            iRegIndex++;
            usNRegs--;
        }
    }
    else
    {
        eStatus = MB_ENOREG;
    }

    return eStatus;
}

eMBErrorCode
eMBRegHoldingCB( uint8_t * pucRegBuffer, uint16_t usAddress, uint16_t usNRegs, eMBRegisterMode eMode )
{
    eMBErrorCode    eStatus = MB_ENOERR;
    int             iRegIndex;

    if( ( usAddress >= REG_HOLDING_START ) && ( usAddress + usNRegs <= REG_HOLDING_START + REG_HOLDING_NREGS ) )
    {
        iRegIndex = ( int )( usAddress - usRegHoldingStart );
        switch ( eMode )
        {
        case MB_REG_READ:
            while( usNRegs > 0 )
            {
                *pucRegBuffer++ = ( unsigned char )( usRegHoldingBuf[iRegIndex] >> 8 );
                *pucRegBuffer++ = ( unsigned char )( usRegHoldingBuf[iRegIndex] & 0xFF );
                iRegIndex++;
                usNRegs--;
            }
            break;

        case MB_REG_WRITE:
            while( usNRegs > 0 )
            {
                usRegHoldingBuf[iRegIndex] = *pucRegBuffer++ << 8;
                usRegHoldingBuf[iRegIndex] |= *pucRegBuffer++;
                iRegIndex++;
                usNRegs--;
            }
        }
    }
    else
    {
        eStatus = MB_ENOREG;
    }
    return eStatus;
}

eMBErrorCode
eMBRegCoilsCB( uint8_t * pucRegBuffer, uint16_t usAddress, uint16_t usNCoils,
               eMBRegisterMode eMode )
{
    return MB_ENOREG;
}

eMBErrorCode
eMBRegDiscreteCB( uint8_t * pucRegBuffer, uint16_t usAddress, uint16_t usNDiscrete )
{
    return MB_ENOREG;
}
